@@ -144,11 +144,12 @@ test_date (void)
 static char *
 test_data_get_data (const char *uri, guint *len)
 {
-	gssize bytes_read;
+	gsize bytes_read;
 	GFileInputStream *stream;
 	GFile *file;
 	GError *error = NULL;
 	char *buffer;
+	gboolean res;
 
 	*len = 0;
 
@@ -175,14 +176,9 @@ test_data_get_data (const char *uri, guint *len)
 	g_object_unref (file);
 
 	buffer = g_malloc (MIME_READ_CHUNK_SIZE);
-	bytes_read = g_input_stream_read (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, NULL, &error);
+	res = g_input_stream_read_all (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, &bytes_read, NULL, &error);
 	g_object_unref (G_INPUT_STREAM (stream));
-	if (bytes_read == -1) {
-		g_free (buffer);
-		return NULL;
-	}
-
-	if (bytes_read == -1) {
+	if (res == FALSE) {
 		g_test_message ("URI '%s' couldn't be read or closed in _get_mime_type_with_data: '%s'", uri, error->message);
 		g_error_free (error);
 		g_free (buffer);
@@ -215,17 +211,18 @@ test_parsability (void)
 	struct {
 		const char *uri;
 		gboolean parsable;
+		gboolean parsable_by_data;
 		gboolean slow;
 	} const files[] = {
 		/* NOTE: For relative paths, don't add a protocol. */
-		{ TEST_SRCDIR "560051.xml", TRUE, FALSE },
-		{ "itms://ax.itunes.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=271121520&ign-mscache=1", TRUE, TRUE },
-		{ "http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=271121520", TRUE, TRUE },
-		{ "file:///tmp/file_doesnt_exist.wmv", FALSE, FALSE },
-		{ "http://live.hujjat.org:7860/main", FALSE, TRUE },
-		{ "http://www.comedycentral.com/sitewide/media_player/videoswitcher.jhtml?showid=934&category=/shows/the_daily_show/videos/headlines&sec=videoId%3D36032%3BvideoFeatureId%3D%3BpoppedFrom%3D_shows_the_daily_show_index.jhtml%3BisIE%3Dfalse%3BisPC%3Dtrue%3Bpagename%3Dmedia_player%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bgateway%3Dshows%3Bsection_1%3Dthe_daily_show%3Bsection_2%3Dvideos%3Bsection_3%3Dheadlines%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bera%3D%27%2Bif_nt_era%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bfla%3D%27%2Bif_nt_Flash%2B%27&itemid=36032&clip=com/dailyshow/headlines/10156_headline.wmv&mswmext=.asx", TRUE, TRUE },
-		{ TEST_SRCDIR "HackerMedley", TRUE, FALSE }, /* From https://bugzilla.redhat.com/show_bug.cgi?id=582850 and http://feeds.feedburner.com/HackerMedley */
-		{ "http://faif.us/feeds/cast-mp3", TRUE, TRUE },
+		{ TEST_SRCDIR "560051.xml", TRUE, TRUE, FALSE },
+		{ "itms://ax.itunes.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=271121520&ign-mscache=1", TRUE, FALSE, TRUE },
+		{ "http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=271121520", TRUE, FALSE, TRUE },
+		{ "file:///tmp/file_doesnt_exist.wmv", FALSE, TRUE, FALSE },
+		{ "http://live.hujjat.org:7860/main", FALSE, TRUE, TRUE },
+		{ "http://www.comedycentral.com/sitewide/media_player/videoswitcher.jhtml?showid=934&category=/shows/the_daily_show/videos/headlines&sec=videoId%3D36032%3BvideoFeatureId%3D%3BpoppedFrom%3D_shows_the_daily_show_index.jhtml%3BisIE%3Dfalse%3BisPC%3Dtrue%3Bpagename%3Dmedia_player%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bgateway%3Dshows%3Bsection_1%3Dthe_daily_show%3Bsection_2%3Dvideos%3Bsection_3%3Dheadlines%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bera%3D%27%2Bif_nt_era%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bfla%3D%27%2Bif_nt_Flash%2B%27&itemid=36032&clip=com/dailyshow/headlines/10156_headline.wmv&mswmext=.asx", TRUE, FALSE, TRUE },
+		{ TEST_SRCDIR "HackerMedley", TRUE, TRUE, FALSE }, /* From https://bugzilla.redhat.com/show_bug.cgi?id=582850 and http://feeds.feedburner.com/HackerMedley */
+		{ "http://faif.us/feeds/cast-mp3", TRUE, TRUE, TRUE },
 		{ NULL,  FALSE, FALSE }
 	};
 
@@ -244,6 +241,10 @@ test_parsability (void)
 		if (files[i].slow && !g_test_slow ())
 			continue;
 
+		/* Skip URIs that aren't available anymore, we can't load data from there... */
+		if (files[i].parsable_by_data == FALSE)
+			continue;
+
 		g_test_message ("Testing data parsing \"%s\"...", files[i].uri);
 
 		data = test_data_get_data (files[i].uri, &len);
@@ -266,6 +267,10 @@ test_parsability (void)
 		/* Slow test! */
 		if (files[i].slow && !g_test_slow ())
 			continue;
+		if (!g_str_has_prefix (files[i].uri, "file://") &&
+		    *files[i].uri != '/') {
+			continue;
+		}
 
 		g_test_message ("Testing filename parsing \"%s\"...", files[i].uri);
 		g_assert (totem_pl_parser_can_parse_from_filename (files[i].uri, TRUE) == files[i].parsable);
@@ -313,31 +318,50 @@ parser_test_get_num_entries (const char *uri)
 	return ret;
 }
 
+typedef struct {
+	gboolean pl_started;
+	gboolean parsed_item;
+	gboolean pl_ended;
+} PlOrderingData;
+
 static void
 playlist_started_order (TotemPlParser *parser,
 			const char *uri,
 			GHashTable *metadata,
-			gboolean *ret)
+			PlOrderingData *data)
 {
-	*ret = TRUE;
+	data->pl_started = TRUE;
+}
+
+static void
+playlist_ended_order (TotemPlParser *parser,
+		      const char *uri,
+		      PlOrderingData *data)
+{
+	g_assert (data->pl_started != FALSE);
+	g_assert (data->parsed_item != FALSE);
+	data->pl_ended = TRUE;
 }
 
 static void
 entry_parsed_cb_order (TotemPlParser *parser,
 		       const char *uri,
 		       GHashTable *metadata,
-		       gboolean *ret)
+		       PlOrderingData *data)
 {
 	/* Check that the playlist started happened before the entry appeared */
-	g_assert (*ret != FALSE);
+	g_assert (data->pl_started != FALSE);
+	data->parsed_item = TRUE;
 }
 
 static gboolean
 parser_test_get_order_result (const char *uri)
 {
 	TotemPlParserResult retval;
-	gboolean pl_started;
-	TotemPlParser *pl = totem_pl_parser_new ();
+	PlOrderingData data;
+	TotemPlParser *pl;
+
+	pl = totem_pl_parser_new ();
 
 	g_object_set (pl, "recurse", !option_no_recurse,
 			  "debug", option_debug,
@@ -345,16 +369,21 @@ parser_test_get_order_result (const char *uri)
 			  "disable-unsafe", option_disable_unsafe,
 			  NULL);
 	g_signal_connect (G_OBJECT (pl), "playlist-started",
-			  G_CALLBACK (playlist_started_order), &pl_started);
+			  G_CALLBACK (playlist_started_order), &data);
+	g_signal_connect (G_OBJECT (pl), "playlist-ended",
+			  G_CALLBACK (playlist_ended_order), &data);
 	g_signal_connect (G_OBJECT (pl), "entry-parsed",
-			  G_CALLBACK (entry_parsed_cb_order), &pl_started);
+			  G_CALLBACK (entry_parsed_cb_order), &data);
 
-	pl_started = FALSE;
+	data.pl_started = FALSE;
+	data.pl_ended = FALSE;
+	data.parsed_item = FALSE;
 	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_assert (data.pl_ended != FALSE);
 	g_test_message ("Got retval %d for uri '%s'", retval, uri);
 	g_object_unref (pl);
 
-	return pl_started;
+	return data.pl_started && data.pl_ended && data.parsed_item;
 }
 
 static TotemPlParserResult
@@ -431,6 +460,40 @@ parser_test_get_playlist_uri (const char *uri)
 	return ret;
 }
 
+static void
+playlist_started_field_cb (TotemPlParser *parser,
+			   const char *uri,
+			   GHashTable *metadata,
+			   ParserResult *res)
+{
+	res->ret = g_strdup (g_hash_table_lookup (metadata, res->field));
+}
+
+static char *
+parser_test_get_playlist_field (const char *uri,
+				const char *field)
+{
+	TotemPlParserResult retval;
+	ParserResult res;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	res.field = field;
+	res.ret = NULL;
+	g_signal_connect (G_OBJECT (pl), "playlist-started",
+			  G_CALLBACK (playlist_started_field_cb), &res);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return res.ret;
+}
+
 static void
 test_image_link (void)
 {
@@ -465,10 +528,22 @@ test_itms_parsing (void)
 		return;
 	}
 
+	/* From https://itunes.apple.com/us/podcast/best-of-chris-moyles/id77318908 */
+	g_assert_cmpstr (parser_test_get_playlist_uri ("itmss://itunes.apple.com/us/podcast/best-of-chris-moyles/id77318908?ign-msr=https%3A%2F%2Fwww.google.com%2Furl%3Fsa%3Dt%26rct%3Dj%26q%3D%26esrc%3Ds%26source%3Dweb%26cd%3D3%26ved%3D0CEEQFjAC%26url%3Dhttps%253A%252F%252Fitunes.apple.com%252Fus%252Fpodcast%252Fbest-of-chris-moyles%252Fid77318908%26ei%3DiMeaUO_ZIsfCtAbvk4DADA%26usg%3DAFQjCNE5YyukECjJE3fmrLIJICX5dMSyyg%26sig2%3DGM631xTbZq7E6m-eaGc3HA"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/moyles/rss.xml");
 	g_assert_cmpstr (parser_test_get_playlist_uri ("itms://itunes.apple.com/gb/podcast/best-of-chris-moyles-enhanced/id142102961?ign-mpt=uo%3D4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/moylesen/rss.xml");
 	g_assert_cmpstr (parser_test_get_playlist_uri ("http://itunes.apple.com/gb/podcast/radio-1-mini-mix/id268491175?uo=4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/r1mix/rss.xml");
 }
 
+static void
+test_pl_content_type (void)
+{
+	char *uri;
+
+	uri = get_relative_uri (TEST_SRCDIR "old-lastfm-output.xspf");
+	g_assert_cmpstr (parser_test_get_playlist_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "application/xspf+xml");
+	g_free (uri);
+}
+
 static void
 test_lastfm_parsing (void)
 {
@@ -512,6 +587,30 @@ test_parsing_xspf_genre (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_xspf_escaping (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "playlist.xspf");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "http://207.200.96.226:8000 extraparam=1");
+	g_free (uri);
+}
+
+static void
+test_parsing_xspf_xml_base (void)
+{
+	char *uri;
+
+	return;
+
+	uri = get_relative_uri (TEST_SRCDIR "xml-base.xspf");
+	/* FIXME: The URL is incorrect here as we're not on HTTP, but
+	 * the parsing is incorrect as well, as we ignore xml:base:
+	 * http://wiki.xiph.org/index.php/XSPF_v1_Notes_and_Errata#xml:base */
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "http://example.org/three/four");
+	g_free (uri);
+}
+
 static void
 test_smi_starttime (void)
 {
@@ -613,6 +712,10 @@ test_parsing_broken_asx (void)
 {
 	TotemPlParserResult result;
 
+	/* FIXME
+	 * URL is gone now */
+	return;
+
 	/* Slow test! */
 	if (!g_test_slow ())
 		return;
@@ -672,6 +775,18 @@ test_parsing_out_of_order_asx (void)
 	g_assert (result != FALSE);
 }
 
+static void
+test_parsing_out_of_order_xspf (void)
+{
+	char *uri;
+	gboolean result;
+
+	uri = get_relative_uri (TEST_SRCDIR "new-lastfm-output.xspf");
+	result = parser_test_get_order_result (uri);
+	g_free (uri);
+	g_assert (result != FALSE);
+}
+
 static void
 test_parsing_num_entries (void)
 {
@@ -771,6 +886,7 @@ test_parsing_xml_mixed_cdata (void)
 	g_free (uri);
 }
 
+#ifdef HAVE_QUVI
 static void
 test_parsing_rss_id (void)
 {
@@ -788,6 +904,7 @@ test_parsing_rss_link (void)
 	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "http://www.guardian.co.uk/technology/audio/2011/may/03/tech-weekly-art-love-bin-laden");
 	g_free (uri);
 }
+#endif /* HAVE_QUVI */
 
 static void
 test_parsing_not_asx_playlist (void)
@@ -938,6 +1055,18 @@ entry_metadata_foreach (const char *key, const char *value, gpointer data)
 		}
 		return;
 	}
+	if (g_ascii_strcasecmp (key, TOTEM_PL_PARSER_FIELD_STARTTIME) == 0) {
+		gint64 res;
+
+		res = totem_pl_parser_parse_duration (value, option_debug);
+		if (res == -1)
+			g_message ("\t%s = '%s' (duration parsing failed)", key, value);
+		else
+			g_message ("\t%s = '%s' (%"G_GINT64_FORMAT" sec)", key, value, res);
+
+		return;
+	}
+
 	g_message ("\t%s = '%s'", key, value);
 }
 
@@ -1124,6 +1253,7 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/parsing_ts_not_ignored", test_parsing_ts_not_ignored);
 		g_test_add_func ("/parser/parsing/mp4_is_flv", test_parsing_mp4_is_flv);
 		g_test_add_func ("/parser/parsing/out_of_order_asx", test_parsing_out_of_order_asx);
+		g_test_add_func ("/parser/parsing/out_of_order_xspf", test_parsing_out_of_order_xspf);
 		g_test_add_func ("/parser/parsing/xml_head_comments", test_parsing_xml_head_comments);
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
 		g_test_add_func ("/parser/parsing/multi_line_rtsptext", test_parsing_rtsp_text_multi);
@@ -1131,13 +1261,18 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/podcast_content_type", test_parsing_content_type);
 		g_test_add_func ("/parser/parsing/live_streaming", test_parsing_live_streaming);
 		g_test_add_func ("/parser/parsing/xml_mixed_cdata", test_parsing_xml_mixed_cdata);
+#ifdef HAVE_QUVI
 		g_test_add_func ("/parser/parsing/rss_id", test_parsing_rss_id);
 		g_test_add_func ("/parser/parsing/rss_link", test_parsing_rss_link);
+#endif /* HAVE_QUVI */
 		g_test_add_func ("/parser/parsing/not_asx_playlist", test_parsing_not_asx_playlist);
 		g_test_add_func ("/parser/parsing/not_really_php", test_parsing_not_really_php);
 		g_test_add_func ("/parser/parsing/not_really_php_but_html_instead", test_parsing_not_really_php_but_html_instead);
 		g_test_add_func ("/parser/parsing/num_items_in_pls", test_parsing_num_entries);
 		g_test_add_func ("/parser/parsing/xspf_genre", test_parsing_xspf_genre);
+		g_test_add_func ("/parser/parsing/xspf_escaping", test_parsing_xspf_escaping);
+		g_test_add_func ("/parser/parsing/xspf_xml_base", test_parsing_xspf_xml_base);
+		g_test_add_func ("/parser/parsing/test_pl_content_type", test_pl_content_type);
 		g_test_add_func ("/parser/parsing/itms_link", test_itms_parsing);
 		g_test_add_func ("/parser/parsing/lastfm-attributes", test_lastfm_parsing);
 		g_test_add_func ("/parser/parsing/m3u_separator", test_m3u_separator);
