@@ -346,6 +346,7 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 	guint i, num_lines;
 	gboolean dos_mode = FALSE;
 	const char *extinfo;
+	char *pl_uri;
 
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -384,6 +385,14 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 	/* We don't count the terminating NULL */
 	num_lines--;
 
+	/* Send out the playlist start and get crackin' */
+	pl_uri = g_file_get_uri (file);
+	totem_pl_parser_add_uri (parser,
+				 TOTEM_PL_PARSER_FIELD_IS_PLAYLIST, TRUE,
+				 TOTEM_PL_PARSER_FIELD_URI, pl_uri,
+				 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, "audio/x-mpegurl",
+				 NULL);
+
 	for (i = 0; lines[i] != NULL; i++) {
 		const char *line;
 
@@ -465,6 +474,9 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 
 	g_strfreev (lines);
 
+	totem_pl_parser_playlist_end (parser, pl_uri);
+	g_free (pl_uri);
+
 	return retval;
 }
 
