@@ -82,6 +82,10 @@ static struct {
 	{ TOTEM_PL_PARSER_FIELD_ALBUM, "album" },
 	{ TOTEM_PL_PARSER_FIELD_DURATION_MS, "duration" },
 	{ TOTEM_PL_PARSER_FIELD_GENRE, NULL },
+	{ TOTEM_PL_PARSER_FIELD_STARTTIME, NULL },
+	{ TOTEM_PL_PARSER_FIELD_SUBTITLE_URI, NULL },
+	{ TOTEM_PL_PARSER_FIELD_PLAYING, NULL },
+	{ TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, NULL }
 };
 
 gboolean
@@ -114,6 +118,7 @@ totem_pl_parser_save_xspf (TotemPlParser    *parser,
 		char *uri, *uri_escaped, *relative;
 		GFile *file;
 		guint i;
+		gboolean wrote_ext;
 
                 totem_pl_playlist_get (playlist, &iter,
                                        TOTEM_PL_PARSER_FIELD_URI, &uri,
@@ -135,6 +140,10 @@ totem_pl_parser_save_xspf (TotemPlParser    *parser,
 		}
 		g_object_unref (file);
 
+		/* Whether we already wrote the GNOME extensions section header
+		 * for that particular track */
+		wrote_ext = FALSE;
+
 		relative = totem_pl_parser_relative (output, uri);
 		uri_escaped = g_markup_escape_text (relative ? relative : uri, -1);
 		buf = g_strdup_printf ("  <track>\n"
@@ -154,22 +163,37 @@ totem_pl_parser_save_xspf (TotemPlParser    *parser,
 			totem_pl_playlist_get (playlist, &iter,
 					       fields[i].field, &str,
 					       NULL);
-			if (!str)
+			if (!str || *str == '\0') {
+				g_free (str);
 				continue;
+			}
 			escaped = g_markup_escape_text (str, -1);
 			g_free (str);
 			if (!escaped)
 				continue;
-			if (g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_GENRE) == FALSE) {
-				buf = g_strdup_printf ("   <%s>%s</%s>\n",
-						       fields[i].element,
-						       escaped,
-						       fields[i].element);
-			} else {
+			if (g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_GENRE)) {
 				buf = g_strdup_printf ("   <extension application=\"http://www.rhythmbox.org\">\n"
 						       "     <genre>%s</genre>\n"
 						       "   </extension>\n",
 						       escaped);
+			} else if (g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_SUBTITLE_URI) ||
+				   g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_PLAYING) ||
+				   g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE) ||
+				   g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_STARTTIME)) {
+				if (!wrote_ext) {
+					buf = g_strdup_printf ("   <extension application=\"http://www.gnome.org\">\n"
+							       "     <%s>%s</%s>\n",
+							       fields[i].field, escaped, fields[i].field);
+					wrote_ext = TRUE;
+				} else {
+					buf = g_strdup_printf ("     <%s>%s</%s>\n",
+							       fields[i].field, escaped, fields[i].field);
+				}
+			} else if (g_str_equal (fields[i].field, TOTEM_PL_PARSER_FIELD_GENRE) == FALSE) {
+				buf = g_strdup_printf ("   <%s>%s</%s>\n",
+						       fields[i].element,
+						       escaped,
+						       fields[i].element);
 			}
 
 			success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
@@ -183,6 +207,12 @@ totem_pl_parser_save_xspf (TotemPlParser    *parser,
                 if (success == FALSE)
 			return FALSE;
 
+		if (wrote_ext)
+			success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), "   </extension>\n", error);
+
+		if (success == FALSE)
+			return FALSE;
+
 		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), "  </track>\n", error);
 		if (success == FALSE)
 			return FALSE;
@@ -200,13 +230,26 @@ totem_pl_parser_save_xspf (TotemPlParser    *parser,
 	return success;
 }
 
+static gboolean
+parse_bool_str (const char *str)
+{
+	if (str == NULL)
+		return FALSE;
+	if (g_ascii_strcasecmp (str, "true") == 0)
+		return TRUE;
+	if (g_ascii_strcasecmp (str, "false") == 0)
+		return FALSE;
+	return atoi (str);
+}
+
 static gboolean
 parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
 	xmlChar *title, *uri, *image_uri, *artist, *album, *duration, *moreinfo;
-	xmlChar *download_uri, *id, *genre, *filesize;
+	xmlChar *download_uri, *id, *genre, *filesize, *subtitle, *mime_type;
+	xmlChar *playing, *starttime;
 	GFile *resolved;
 	char *resolved_uri;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
@@ -222,6 +265,10 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 	id = NULL;
 	genre = NULL;
 	filesize = NULL;
+	subtitle = NULL;
+	mime_type = NULL;
+	playing = NULL;
+	starttime = NULL;
 
 	for (node = parent->children; node != NULL; node = node->next)
 	{
@@ -266,6 +313,23 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 						break;
 					}
 				}
+			} else if (app != NULL && g_ascii_strcasecmp ((char *) app, "http://www.gnome.org") == 0) {
+				xmlNodePtr child;
+				for (child = node->xmlChildrenNode ; child; child = child->next) {
+					if (child->name == NULL)
+						continue;
+					if (g_ascii_strcasecmp ((char *)child->name, "playing") == 0) {
+						xmlChar *str;
+						str = xmlNodeListGetString (doc, child->xmlChildrenNode, 0);
+						playing = parse_bool_str ((char *) str) ? (xmlChar *) "true" : NULL;
+					} else if (g_ascii_strcasecmp ((char *)child->name, "subtitle") == 0) {
+						subtitle = xmlNodeListGetString (doc, child->xmlChildrenNode, 0);
+					} else if (g_ascii_strcasecmp ((char *)child->name, "mime-type") == 0) {
+						mime_type = xmlNodeListGetString (doc, child->xmlChildrenNode, 0);
+					} else if (g_ascii_strcasecmp ((char *)child->name, "starttime") == 0) {
+						starttime = xmlNodeListGetString (doc, child->xmlChildrenNode, 0);
+					}
+				}
 			} else if (app != NULL && g_ascii_strcasecmp ((char *) app, "http://www.last.fm") == 0) {
 				xmlNodePtr child;
 				for (child = node->xmlChildrenNode ; child; child = child->next) {
@@ -306,23 +370,49 @@ parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 	}
 
 	resolved_uri = totem_pl_parser_resolve_uri (base_file, (char *) uri);
-	resolved = g_file_new_for_uri (resolved_uri);
-	g_free (resolved_uri);
 
-	totem_pl_parser_add_uri (parser,
-				 TOTEM_PL_PARSER_FIELD_FILE, resolved,
-				 TOTEM_PL_PARSER_FIELD_TITLE, title,
-				 TOTEM_PL_PARSER_FIELD_DURATION_MS, duration,
-				 TOTEM_PL_PARSER_FIELD_IMAGE_URI, image_uri,
-				 TOTEM_PL_PARSER_FIELD_AUTHOR, artist,
-				 TOTEM_PL_PARSER_FIELD_ALBUM, album,
-				 TOTEM_PL_PARSER_FIELD_MOREINFO, moreinfo,
-				 TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI, download_uri,
-				 TOTEM_PL_PARSER_FIELD_ID, id,
-				 TOTEM_PL_PARSER_FIELD_GENRE, genre,
-				 TOTEM_PL_PARSER_FIELD_FILESIZE, filesize,
-				 NULL);
-	g_object_unref (resolved);
+	if (g_strcmp0 (resolved_uri, (char *) uri) == 0) {
+		g_free (resolved_uri);
+		totem_pl_parser_add_uri (parser,
+					 TOTEM_PL_PARSER_FIELD_URI, uri,
+					 TOTEM_PL_PARSER_FIELD_TITLE, title,
+					 TOTEM_PL_PARSER_FIELD_DURATION_MS, duration,
+					 TOTEM_PL_PARSER_FIELD_IMAGE_URI, image_uri,
+					 TOTEM_PL_PARSER_FIELD_AUTHOR, artist,
+					 TOTEM_PL_PARSER_FIELD_ALBUM, album,
+					 TOTEM_PL_PARSER_FIELD_MOREINFO, moreinfo,
+					 TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI, download_uri,
+					 TOTEM_PL_PARSER_FIELD_ID, id,
+					 TOTEM_PL_PARSER_FIELD_GENRE, genre,
+					 TOTEM_PL_PARSER_FIELD_FILESIZE, filesize,
+					 TOTEM_PL_PARSER_FIELD_SUBTITLE_URI, subtitle,
+					 TOTEM_PL_PARSER_FIELD_PLAYING, playing,
+					 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, mime_type,
+					 TOTEM_PL_PARSER_FIELD_STARTTIME, starttime,
+					 NULL);
+	} else {
+		resolved = g_file_new_for_uri (resolved_uri);
+		g_free (resolved_uri);
+
+		totem_pl_parser_add_uri (parser,
+					 TOTEM_PL_PARSER_FIELD_FILE, resolved,
+					 TOTEM_PL_PARSER_FIELD_TITLE, title,
+					 TOTEM_PL_PARSER_FIELD_DURATION_MS, duration,
+					 TOTEM_PL_PARSER_FIELD_IMAGE_URI, image_uri,
+					 TOTEM_PL_PARSER_FIELD_AUTHOR, artist,
+					 TOTEM_PL_PARSER_FIELD_ALBUM, album,
+					 TOTEM_PL_PARSER_FIELD_MOREINFO, moreinfo,
+					 TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI, download_uri,
+					 TOTEM_PL_PARSER_FIELD_ID, id,
+					 TOTEM_PL_PARSER_FIELD_GENRE, genre,
+					 TOTEM_PL_PARSER_FIELD_FILESIZE, filesize,
+					 TOTEM_PL_PARSER_FIELD_SUBTITLE_URI, subtitle,
+					 TOTEM_PL_PARSER_FIELD_PLAYING, playing,
+					 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, mime_type,
+					 TOTEM_PL_PARSER_FIELD_STARTTIME, starttime,
+					 NULL);
+		g_object_unref (resolved);
+	}
 
 	retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
@@ -362,19 +452,53 @@ parse_xspf_trackList (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 }
 
 static gboolean
-parse_xspf_entries (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
-		xmlNodePtr parent)
+parse_xspf_entries (TotemPlParser *parser,
+		    GFile         *file,
+		    GFile         *base_file,
+		    xmlDocPtr      doc,
+		    xmlNodePtr     parent)
 {
 	xmlNodePtr node;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
+	const xmlChar *title;
+	char *uri;
+
+	uri = g_file_get_uri (file);
+	title = NULL;
+
+	/* We go through the list twice to avoid the playlist-started
+	 * signal being out of order */
 
 	for (node = parent->children; node != NULL; node = node->next) {
 		if (node->name == NULL)
 			continue;
 
-		if (g_ascii_strcasecmp ((char *)node->name, "trackList") == 0)
+		if (g_ascii_strcasecmp ((char *)node->name, "title") == 0) {
+			title = node->name;
+			break;
+		}
+	}
+
+	totem_pl_parser_add_uri (parser,
+				 TOTEM_PL_PARSER_FIELD_IS_PLAYLIST, TRUE,
+				 TOTEM_PL_PARSER_FIELD_URI, uri,
+				 TOTEM_PL_PARSER_FIELD_TITLE, title,
+				 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, "application/xspf+xml",
+				 NULL);
+
+	for (node = parent->children; node != NULL; node = node->next) {
+		if (node->name == NULL)
+			continue;
+
+		if (g_ascii_strcasecmp ((char *)node->name, "trackList") == 0) {
 			if (parse_xspf_trackList (parser, base_file, doc, node) != FALSE)
 				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
+		}
+	}
+
+	if (uri != NULL) {
+		totem_pl_parser_playlist_end (parser, uri);
+		g_free (uri);
 	}
 
 	return retval;
@@ -415,7 +539,7 @@ totem_pl_parser_add_xspf_with_contents (TotemPlParser *parser,
 	}
 
 	for (node = doc->children; node != NULL; node = node->next) {
-		if (parse_xspf_entries (parser, base_file, doc, node) != FALSE)
+		if (parse_xspf_entries (parser, file, base_file, doc, node) != FALSE)
 			retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
@@ -442,7 +566,7 @@ totem_pl_parser_add_xspf (TotemPlParser *parser,
 	}
 
 	for (node = doc->children; node != NULL; node = node->next) {
-		if (parse_xspf_entries (parser, base_file, doc, node) != FALSE)
+		if (parse_xspf_entries (parser, file, base_file, doc, node) != FALSE)
 			retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
